@@ -12,6 +12,7 @@
 * directly using the email address security@modsecurity.org.
 */
 
+#include <assert.h>
 #include "msc_geo.h"
 
 
@@ -244,6 +245,7 @@ static int field_length(const char *field, int maxlen)
  */
 int geo_init(directory_config *dcfg, const char *dbfn, char **error_msg)
 {
+    assert(dcfg != NULL);
     *error_msg = NULL;
 
     if ((dcfg->geo == NULL) || (dcfg->geo == NOT_SET_P)) {
@@ -263,6 +265,10 @@ int geo_init(directory_config *dcfg, const char *dbfn, char **error_msg)
  */
 int geo_lookup(modsec_rec *msr, geo_rec *georec, const char *target, char **error_msg)
 {
+    assert(msr != NULL);
+    assert(georec != NULL);
+    assert(target != NULL);
+    assert(error_msg != NULL);
     apr_sockaddr_t *addr;
     long ipnum = 0;
     char *targetip = NULL;
