@@ -14,6 +14,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <apr.h>
 #if APR_HAVE_STDINT_H
 #include <stdint.h>
 #endif
@@ -534,11 +535,13 @@ int TreeCheckData(TreePrefix *prefix, CPTData *prefix_data, unsigned int netmask
 }
 
 int TreePrefixNetmask(modsec_rec *msr, TreePrefix *prefix, unsigned int netmask, int flag)   {
+    // msr can be NULL;
+    assert(!msr || msr->txcfg != NULL);
     CPTData *prefix_data = NULL;
     int ret = 0;
 
     if (prefix == NULL) {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "TreePrefixNetmask: prefix is NULL.");
         }
         return 0;
@@ -551,19 +554,19 @@ int TreePrefixNetmask(modsec_rec *msr, TreePrefix *prefix, unsigned int netmask,
         if(prefix_data == NULL) return 0;
 
         if (prefix_data->netmask != netmask) {
-            if (msr->txcfg->debuglog_level >= 9) {
+            if (msr && msr->txcfg->debuglog_level >= 9) {
                 msr_log(msr, 9, "TreePrefixNetmask: Cannot find a prefix with correct netmask.");
             }
             return 0;
         } else {
-            if (msr->txcfg->debuglog_level >= 9) {
+            if (msr && msr->txcfg->debuglog_level >= 9) {
                 msr_log(msr, 9, "TreePrefixNetmask: Found a prefix with correct netmask.");
             }
             return 1;
         }
     }
 
-    if (msr->txcfg->debuglog_level >= 9) {
+    if (msr && msr->txcfg->debuglog_level >= 9) {
         msr_log(msr, 9, "TreePrefixNetmask: Check if a prefix has a the correct netmask");
     }
 
@@ -573,17 +576,19 @@ int TreePrefixNetmask(modsec_rec *msr, TreePrefix *prefix, unsigned int netmask,
 }
 
 TreeNode *CPTRetriveNode(modsec_rec *msr, unsigned char *buffer, unsigned int ip_bitmask, TreeNode *node)  {
+    // msr can be NULL;
+    assert(!msr || msr->txcfg != NULL);
     unsigned int x, y;
 
     if(node == NULL)    {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTRetriveNode: Node tree is NULL.");
         }
         return NULL;
     }
 
     if(buffer == NULL)  {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTRetriveNode: Empty ip address. Nothing to search for.");
         }
         return NULL;
@@ -602,7 +607,7 @@ TreeNode *CPTRetriveNode(modsec_rec *msr, unsigned char *buffer, unsigned int ip
         }
     }
 
-    if (msr->txcfg->debuglog_level >= 9) {
+    if (msr && msr->txcfg->debuglog_level >= 9) {
         msr_log(msr, 9, "CPTRetriveNode: Found the node for provided ip address.");
     }
 
@@ -619,6 +624,8 @@ TreeNode *CPTRetriveParentNode(TreeNode *node)  {
 }
 
 TreeNode *CPTFindElementIPNetblock(modsec_rec *msr, unsigned char *ipdata, unsigned char ip_bitmask, TreeNode *node) {
+    // msr can be NULL;
+    assert(!msr || msr->txcfg != NULL);
     TreeNode *netmask_node = NULL;
     int mask = 0,  bytes = 0;
     int i = 0, j = 0;
@@ -627,7 +634,7 @@ TreeNode *CPTFindElementIPNetblock(modsec_rec *msr, unsigned char *ipdata, unsig
     node = CPTRetriveParentNode(node);
 
     if (node == NULL)   {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTFindElementIPNetblock: Node tree is NULL.");
         }
         return NULL;
@@ -655,16 +662,23 @@ TreeNode *CPTFindElementIPNetblock(modsec_rec *msr, unsigned char *ipdata, unsig
         }
 
         node = CPTRetriveNode(msr, ipdata, ip_bitmask, node);
+        if (!node) {
+            if (msr && msr->txcfg->debuglog_level >= 9) {
+                msr_log(msr, 9, "CPTFindElementIPNetblock: No tree node found.");
+            }
+            return NULL;
+        }
 
-        if (node && node->bit != ip_bitmask)    {
-            if (msr->txcfg->debuglog_level >= 9) {
+
+        if (node->bit != ip_bitmask)    {
+            if (msr && msr->txcfg->debuglog_level >= 9) {
                 msr_log(msr, 9, "CPTFindElementIPNetblock: Found a tree node but netmask is different.");
             }
             return NULL;
         }
 
-        if (node && node->prefix == NULL)   {
-            if (msr->txcfg->debuglog_level >= 9) {
+        if (node->prefix == NULL)   {
+            if (msr && msr->txcfg->debuglog_level >= 9) {
                 msr_log(msr, 9, "CPTFindElementIPNetblock: Found a tree node but prefix is NULL.");
             }
             return NULL;
@@ -675,7 +689,7 @@ TreeNode *CPTFindElementIPNetblock(modsec_rec *msr, unsigned char *ipdata, unsig
 
             if ((ip_bitmask % 8) == 0) {
                 if (TreePrefixNetmask(msr, node->prefix, netmask_node->netmasks[j], FALSE)) {
-                    if (msr->txcfg->debuglog_level >= 9) {
+                    if (msr && msr->txcfg->debuglog_level >= 9) {
                         msr_log(msr, 9, "CPTFindElementIPNetblock: Node found for provided ip address");
                     }
                     return node;
@@ -684,7 +698,7 @@ TreeNode *CPTFindElementIPNetblock(modsec_rec *msr, unsigned char *ipdata, unsig
 
             if ((node->prefix->buffer[bytes] & mask) == (ipdata[bytes] & mask)) {
                 if (TreePrefixNetmask(msr, node->prefix, netmask_node->netmasks[j], FALSE)) {
-                    if (msr->txcfg->debuglog_level >= 9) {
+                    if (msr && msr->txcfg->debuglog_level >= 9) {
                         msr_log(msr, 9, "CPTFindElementIPNetblock: Node found for provided ip address");
                     }
                     return node;
@@ -699,19 +713,21 @@ TreeNode *CPTFindElementIPNetblock(modsec_rec *msr, unsigned char *ipdata, unsig
 }
 
 TreeNode *CPTFindElement(modsec_rec *msr, unsigned char *ipdata, unsigned int ip_bitmask, CPTTree *tree)   {
+    // msr can be NULL;
+    assert(!msr || msr->txcfg != NULL);
     TreeNode *node = NULL;
     int mask = 0, bytes = 0;
     unsigned char temp_data[NETMASK_256-1];
 
     if (tree == NULL)   {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTFindElement: Tree is NULL. Cannot proceed searching the ip.");
         }
         return node;
     }
 
     if (tree->head == NULL) {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTFindElement: Tree head is NULL. Cannot proceed searching the ip.");
         }
         return node;
@@ -720,7 +736,7 @@ TreeNode *CPTFindElement(modsec_rec *msr, unsigned char *ipdata, unsigned int ip
     node = tree->head;
 
     if (ip_bitmask > (NETMASK_256-1))   {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTFindElement: Netmask cannot be greater than 255");
         }
         return NULL;
@@ -734,32 +750,32 @@ TreeNode *CPTFindElement(modsec_rec *msr, unsigned char *ipdata, unsigned int ip
     node = CPTRetriveNode(msr, temp_data, ip_bitmask, node);
 
     if (node && (node->bit != ip_bitmask)) {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTFindElement: Found a tree node but netmask is different.");
         }
         return NULL;
     }
 
     if(node == NULL)    {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTFindElement: Node tree is NULL.");
         }
         return node;
     }
 
     if(node->prefix == NULL)    {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTFindElement: Found a tree node but prefix is NULL.");
         }
         return node;
     }
 
-    if (memcmp(node->prefix->buffer, temp_data, bytes) == 0) {
+    if ((node->netmasks == NULL) && (memcmp(node->prefix->buffer, temp_data, bytes) == 0)) {
         mask = SHIFT_LEFT_MASK(8 - ip_bitmask % 8);
 
         if ((ip_bitmask % 8) == 0) {
             if (TreePrefixNetmask(msr, node->prefix, ip_bitmask, TRUE)) {
-                if (msr->txcfg->debuglog_level >= 9) {
+                if (msr && msr->txcfg->debuglog_level >= 9) {
                     msr_log(msr, 9, "CPTFindElement: Node found for provided ip address");
                 }
                 return node;
@@ -768,7 +784,7 @@ TreeNode *CPTFindElement(modsec_rec *msr, unsigned char *ipdata, unsigned int ip
 
         if ((node->prefix->buffer[bytes] & mask) == (temp_data[bytes] & mask)) {
             if (TreePrefixNetmask(msr, node->prefix, ip_bitmask, TRUE)) {
-                if (msr->txcfg->debuglog_level >= 9) {
+                if (msr && msr->txcfg->debuglog_level >= 9) {
                     msr_log(msr, 9, "CPTFindElement: Node found for provided ip address");
                 }
                 return node;
@@ -780,16 +796,18 @@ TreeNode *CPTFindElement(modsec_rec *msr, unsigned char *ipdata, unsigned int ip
 }
 
 TreeNode *CPTIpMatch(modsec_rec *msr, unsigned char *ipdata, CPTTree *tree, int type)   {
+    // msr can be NULL;
+    assert(!msr || msr->txcfg != NULL);
 
     if(tree == NULL)  {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTIpMatch: Tree is NULL. Cannot proceed searching the ip.");
         }
         return NULL;
     }
 
     if(ipdata == NULL)  {
-        if (msr->txcfg->debuglog_level >= 9) {
+        if (msr && msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "CPTIpMatch: Empty ip address. Nothing to search for.");
         }
         return NULL;
@@ -797,17 +815,17 @@ TreeNode *CPTIpMatch(modsec_rec *msr, unsigned char *ipdata, CPTTree *tree, int
 
     switch(type)    {
         case IPV4_TREE:
-            if (msr->txcfg->debuglog_level >= 9) {
+            if (msr && msr->txcfg->debuglog_level >= 9) {
                 msr_log(msr, 9, "CPTIpMatch: Searching ip type 0x%x", type);
             }
             return CPTFindElement(msr, ipdata, NETMASK_32, tree);
         case IPV6_TREE:
-            if (msr->txcfg->debuglog_level >= 9) {
+            if (msr && msr->txcfg->debuglog_level >= 9) {
                 msr_log(msr, 9, "CPTIpMatch: Searching ip type 0x%x", type);
             }
             return CPTFindElement(msr, ipdata, NETMASK_128, tree);
         default:
-            if (msr->txcfg->debuglog_level >= 9) {
+            if (msr && msr->txcfg->debuglog_level >= 9) {
                 msr_log(msr, 9, "CPTIpMatch: Unknown ip type 0x%x", type);
             }
             return NULL;
@@ -820,73 +838,92 @@ TreeNode *TreeAddIP(const char *buffer, CPTTree *tree, int type) {
     char ip_strv4[NETMASK_32], ip_strv6[NETMASK_128];
     struct in_addr addr4;
     struct in6_addr addr6;
+    int pos = 0;
     char *ptr = NULL;
 
     if(tree == NULL)
         return NULL;
 
+    pos = strchr(buffer, '/') - buffer;
+
     switch(type)    {
 
         case IPV4_TREE:
-            memset(&addr4, 0, sizeof(addr4));
+            memset(&(addr4.s_addr), 0, sizeof(addr4.s_addr));
             memset(ip_strv4, 0x0, NETMASK_32);
 
-            strncpy(ip_strv4, buffer, sizeof(ip_strv4) - 2);
+            strncpy(ip_strv4, buffer, sizeof(ip_strv4));
             *(ip_strv4 + (sizeof(ip_strv4) - 1)) = '\0';
 
             ptr = strdup(ip_strv4);
+            if (ptr == NULL) return NULL; // No way to return a clean error message
             netmask_v4 = is_netmask_v4(ptr);
 
+            if (netmask_v4 > NETMASK_32) {
+                free(ptr);
+                ptr = NULL;
+                return NULL;
+            }
+
             if(ptr != NULL) {
                 free(ptr);
                 ptr = NULL;
             }
 
-            if(netmask_v4 == 0)
+            if(netmask_v4 == 0) {
                 return NULL;
-            else if(netmask_v4 != NETMASK_32)   {
-                ip_strv4[strlen(ip_strv4)-3] = '\0';
+            }
+            else if (netmask_v4 != NETMASK_32 && pos < strlen(ip_strv4)) {
+                ip_strv4[pos] = '\0';
             }
 
-            ret = inet_pton(AF_INET, ip_strv4, &addr4);
+            ret = inet_pton(AF_INET, ip_strv4, &(addr4.s_addr));
+
             if (ret <= 0) {
                 return NULL;
             }
-
-            ip = addr4.s_addr;
-
             tree->count++;
-
-            return CPTAddElement((unsigned char *)&ip, NETMASK_32, tree, netmask_v4);
+            return CPTAddElement((unsigned char *)&(addr4.s_addr), NETMASK_32, tree, netmask_v4);
 
         case IPV6_TREE:
-            memset(&addr6, 0, sizeof(addr6));
+            memset(&(addr6.s6_addr), 0, sizeof(addr6.s6_addr));
             memset(ip_strv6, 0x0, NETMASK_128);
 
-            strncpy(ip_strv6, buffer, sizeof(ip_strv6) - 2);
+            strncpy(ip_strv6, buffer, sizeof(ip_strv6));
             *(ip_strv6 + sizeof(ip_strv6) - 1) = '\0';
 
             ptr = strdup(ip_strv6);
+            if (ptr == NULL) return NULL; // No way to return a clean error message
             netmask_v6 = is_netmask_v6(ptr);
 
+            if (netmask_v6 > NETMASK_128) {
+                free(ptr);
+                ptr = NULL;
+                return NULL;
+            }
+
             if(ptr != NULL) {
                 free(ptr);
                 ptr = NULL;
             }
 
-            if(netmask_v6 == 0)
+            if(netmask_v6 == 0) {
                 return NULL;
-            else if (netmask_v6 != NETMASK_64)   {
-                ip_strv4[strlen(ip_strv6)-3] = '\0';
+            }
+            else if (netmask_v6 != NETMASK_128 && pos < strlen(ip_strv6)) {
+                ip_strv6[pos] = '\0';
             }
 
-            ret = inet_pton(AF_INET6, ip_strv6, &addr6);
-            if(ret <= 0)
+            ret = inet_pton(AF_INET6, ip_strv6, &(addr6.s6_addr));
+
+            if (ret <= 0)
+            {
                 return NULL;
+            }
 
             tree->count++;
 
-            return CPTAddElement((unsigned char *)&addr6.s6_addr, NETMASK_128, tree, netmask_v6);
+            return CPTAddElement((unsigned char *)&(addr6.s6_addr), NETMASK_128, tree, netmask_v6);
         default:
             return NULL;
     }
