@@ -667,6 +667,7 @@ int convert_to_int(const char c)
  * \retval 0 On Sucess|Fail
  */
 int set_match_to_tx(modsec_rec *msr, int capture, const char *match, int tx_n)  {
+    assert(msr != NULL);
 
     if (capture) {
         msc_string *s = (msc_string *)apr_pcalloc(msr->mp, sizeof(msc_string));
@@ -2378,6 +2379,7 @@ apr_fileperms_t mode2fileperms(int mode) {
  * Generate a single variable.
  */
 char *construct_single_var(modsec_rec *msr, char *name) {
+    assert(msr != NULL);
     char *varname = NULL;
     char *param = NULL;
     msre_var *var = NULL;
@@ -2386,6 +2388,7 @@ char *construct_single_var(modsec_rec *msr, char *name) {
 
     /* Extract variable name and its parameter from the script. */
     varname = apr_pstrdup(msr->mp, name);
+    if (varname == NULL) return NULL;
     param = strchr(varname, '.');
     if (param != NULL) {
         *param = '\0';
@@ -2470,28 +2473,16 @@ not_enough_memory:
 
 int read_line(char *buf, int len, FILE *fp)
 {
-    char *tmp;
+    if (buf == NULL) return -1;
 
-    if (buf == NULL)
-    {
-        return -1;
-    }
-
-    memset(buf, '\0', len*sizeof(char));
-
-    if (fgets(buf, len, fp) == NULL)
-    {
+    if (fgets(buf, len, fp) == NULL) {
         *buf = '\0';
         return 0;
     }
-    else
-    {
-        if ((tmp = strrchr(buf, '\n')) != NULL)
-        {
-            *tmp = '\0';
-        }
-    }
-
+    
+    char* tmp;
+    if ((tmp = strrchr(buf, '\n')) != NULL) *tmp = '\0';
+    
     return 1;
 }
 
@@ -2703,6 +2694,10 @@ int ip_tree_from_uri(TreeRoot **rtree, char *uri,
 int tree_contains_ip(apr_pool_t *mp, TreeRoot *rtree,
     const char *value, modsec_rec *msr, char **error_msg)
 {
+    assert(mp != NULL);
+    assert(value != NULL);
+    // msr can be NULL;
+    assert(error_msg != NULL);
     struct in_addr in;
 #if APR_HAVE_IPV6
     struct in6_addr in6;
@@ -2843,3 +2838,14 @@ char* strtok_r(
 }
 #endif
 
+// we cannot log an error message as this happens much too often
+char* get_username(apr_pool_t* mp) {
+    char* username;
+    apr_uid_t uid;
+    apr_gid_t gid;
+    int rc = apr_uid_current(&uid, &gid, mp);
+    if (rc != APR_SUCCESS) return "apache";
+    rc = apr_uid_name_get(&username, uid, mp);
+    if (rc != APR_SUCCESS) return "apache";
+    return username;
+}
