@@ -1,6 +1,6 @@
 /*
 * ModSecurity for Apache 2.x, http://www.modsecurity.org/
-* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+* Copyright (c) 2004-2013 Trustwave Holdings, Inc. (http://www.trustwave.com/)
 *
 * You may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
@@ -432,6 +432,7 @@ AP_DECLARE(apr_status_t) ap_pcfg_openfile(ap_configfile_t **ret_cfg,
     apr_file_t *file = NULL;
     apr_finfo_t finfo;
     apr_status_t status;
+    int exist_type;
 #ifdef DEBUG
     char buf[120];
 #endif
@@ -457,13 +458,13 @@ AP_DECLARE(apr_status_t) ap_pcfg_openfile(ap_configfile_t **ret_cfg,
     if (status != APR_SUCCESS)
         return status;
 
-    if (finfo.filetype != APR_REG &&
+    exist_type = (finfo.filetype != APR_REG);
 #if defined(WIN32) || defined(OS2) || defined(NETWARE)
-        strcasecmp(apr_filepath_name_get(name), "nul") != 0) {
+    exist_type = (exist_type && strcasecmp(apr_filepath_name_get(name), "nul") != 0);
 #else
-        strcmp(name, "/dev/null") != 0) {
+    exist_type = (exist_type && strcmp(name, "/dev/null") != 0);
 #endif /* WIN32 || OS2 */
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, NULL,
+    if (exist_type){ ap_log_error(APLOG_MARK, APLOG_ERR, 0, NULL,
                      "Access to file %s denied by server: not a regular file",
                      name);
         apr_file_close(file);
@@ -503,7 +504,7 @@ AP_DECLARE(apr_status_t) ap_pcfg_openfile(ap_configfile_t **ret_cfg,
 #else
     new_cfg->getch = cfg_getch;
     new_cfg->getstr = cfg_getstr;
-    new_cfg->close = cfg_close; 
+    new_cfg->close = cfg_close;
 #endif
     new_cfg->line_number = 0;
     *ret_cfg = new_cfg;
@@ -741,12 +742,43 @@ AP_DECLARE(char *) ap_make_full_path(apr_pool_t *a, const char *src1,
     return path;
 }
 
-static int fname_alphasort(const void *fn1, const void *fn2)
+static int fname_reversealphasort(const void *fn1, const void *fn2)
 {
     const fnames *f1 = fn1;
     const fnames *f2 = fn2;
 
-    return strcmp(f1->fname,f2->fname);
+    return strcmp(f2->fname,f1->fname);
+}
+
+int fnmatch_test(const char *pattern)
+{
+    int nesting;
+
+    nesting = 0;
+    while (*pattern) {
+        switch (*pattern) {
+        case '?':
+        case '*':
+            return 1;
+
+/*        case '\\':
+            if (*++pattern == '\0') {
+                return 0;
+            }
+            break;*/		// this breaks on Windows
+
+        case '[':         /* '[' is only a glob if it has a matching ']' */
+            ++nesting;
+            break;
+
+        case ']':
+            if (nesting) {
+                return 1;
+            }
+            break;
+        }
+        ++pattern;    }
+    return 0;
 }
 
 AP_DECLARE(const char *) process_resource_config(const char *fname,
@@ -808,7 +840,7 @@ static const char *process_resource_config_nofnmatch(const char *fname,
         apr_dir_close(dirp);
         if (candidates->nelts != 0) {
             qsort((void *) candidates->elts, candidates->nelts,
-                  sizeof(fnames), fname_alphasort);
+                  sizeof(fnames), fname_reversealphasort);
 
             /*
              * Now recurse these... we handle errors and subdirectories
@@ -859,7 +891,7 @@ static const char *process_resource_config_fnmatch(const char *path,
     }
 
     /* optimisation - if the filename isn't a wildcard, process it directly */
-    if (!apr_fnmatch_test(fname)) {
+    if (!fnmatch_test(fname)) {
         path = ap_make_full_path(ptemp, path, fname);
         if (!rest) {
             return process_resource_config_nofnmatch(path,
@@ -891,7 +923,7 @@ static const char *process_resource_config_fnmatch(const char *path,
         if (strcmp(dirent.name, ".")
             && strcmp(dirent.name, "..")
             && (apr_fnmatch(fname, dirent.name,
-                            APR_FNM_PERIOD) == APR_SUCCESS)) {
+                            APR_FNM_PERIOD | APR_FNM_NOESCAPE | APR_FNM_PATHNAME) == APR_SUCCESS)) {
             const char *full_path = ap_make_full_path(ptemp, path, dirent.name);
             /* If matching internal to path, and we happen to match something
              * other than a directory, skip it
@@ -909,7 +941,7 @@ static const char *process_resource_config_fnmatch(const char *path,
         const char *error;
 
         qsort((void *) candidates->elts, candidates->nelts,
-              sizeof(fnames), fname_alphasort);
+              sizeof(fnames), fname_reversealphasort);
 
         /*
          * Now recurse these... we handle errors and subdirectories
@@ -949,7 +981,7 @@ AP_DECLARE(const char *) process_fnmatch_configs(apr_array_header_t *ari,
                                                     apr_pool_t *ptemp,
                                                     int optional)
 {
-    if (!apr_fnmatch_test(fname)) {
+    if (!fnmatch_test(fname)) {
         return process_resource_config(fname, ari, p);
     }
     else {
@@ -996,6 +1028,8 @@ const char *process_command_config(server_rec *s,
 	ap_directive_t *newdir;
 	int optional;
 	char *err = NULL;
+	char *rootpath, *incpath;
+	int li;
 
 	errmsg = populate_include_files(p, ptemp, ari, filename, 0);
 
@@ -1067,6 +1101,27 @@ ProcessInclude:
 					goto Exit;
 				}
 
+				incpath = w;
+
+				/* locate the start of the directories proper */
+				status = apr_filepath_root(&rootpath, &incpath, APR_FILEPATH_TRUENAME | APR_FILEPATH_NATIVE, ptemp);
+
+				/* we allow APR_SUCCESS and APR_EINCOMPLETE */
+				if (APR_ERELATIVE == status) {
+					rootpath = apr_pstrdup(ptemp, parms->config_file->name);
+					li = strlen(rootpath) - 1;
+
+					while(li >= 0 && rootpath[li] != '/' && rootpath[li] != '\\')
+						rootpath[li--] = 0;
+
+					w = apr_pstrcat(p, rootpath, w, NULL);
+				}
+				else if (APR_EBADPATH == status) {
+					ap_cfg_closefile(parms->config_file);
+					errmsg = apr_pstrcat(p, "Include file has a bad path, ", w, NULL);
+					goto Exit;
+				}
+
 				errmsg = populate_include_files(p, ptemp, ari, w, optional);
 
 				*(cmd_parms *)apr_array_push(arr) = *parms;
@@ -1099,7 +1154,21 @@ ProcessInclude:
 
 			parms->directive = newdir;
 
-			errmsg = invoke_cmd(cmd, parms, mconfig, args);
+#ifdef WIN32
+			// some config commands fail in APR when there are file
+			// permission issues or other OS-specific problems
+			//
+			__try
+			{
+#endif
+				errmsg = invoke_cmd(cmd, parms, mconfig, args);
+#ifdef WIN32
+			}
+			__except(EXCEPTION_EXECUTE_HANDLER)
+			{
+				errmsg = "Command failed to execute (check file/folder permissions, syntax, etc.).";
+			}
+#endif
 
 			if(errmsg != NULL)
 				break;
@@ -1132,3 +1201,4 @@ Exit:
 
 	return errmsg;
 }
+
